@@ -34,7 +34,16 @@ Georeferencing
    pixel_coordinates
    get_earth_radius
    get_radolan_grid
+   reproject_raster_dataset
    resample_raster_dataset
+   get_shape_coordinates
+   correct_parallax
+   sat2pol
+   dist_from_orbit
+   create_raster_dataset
+   set_raster_origin
+   extract_raster_dataset
+
 
 """
 
@@ -51,11 +60,13 @@ Georeferencing
 # a - azimuth (von süden aus gezählt)
 # h - Höhe über Horizont
 
-from osgeo import gdal, osr
+from osgeo import gdal, osr, gdal_array
 import numpy as np
 from sys import exit
 import warnings
 
+from . import util as util
+
 
 def hor2aeq(a, h, phi):
     """"""
@@ -91,10 +102,10 @@ def polar2lonlat(r, az, sitecoords, re=6370040):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]
-    az : array
-        array of azimuth angles containing values between 0° and 360°.
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]
+    az : :class:`numpy:numpy.ndarray`
+        Array of azimuth angles containing values between 0° and 360°.
         These are assumed to start with 0° pointing north and counted positive
         clockwise!
     sitecoords : a sequence of two floats
@@ -105,7 +116,8 @@ def polar2lonlat(r, az, sitecoords, re=6370040):
     Returns
     -------
     lon, lat : tuple
-        two arrays containing the spherical longitude and latitude coordinates
+        Tuple of two :class:`numpy:numpy.ndarray` containing the spherical
+        longitude and latitude coordinates.
 
     Note
     ----
@@ -224,10 +236,10 @@ def polar2lonlatalt(r, az, elev, sitecoords, re=6370040.):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]
-    az : array
-        array of azimuth angles containing values between 0° and 360°.
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]
+    az : :class:`numpy:numpy.ndarray`
+        Array of azimuth angles containing values between 0° and 360°.
         These are assumed to start with 0° pointing north and counted positive
         clockwise!
     sitecoords : a sequence of three floats
@@ -240,7 +252,8 @@ def polar2lonlatalt(r, az, elev, sitecoords, re=6370040.):
     Returns
     -------
     output : tuple
-        three arrays (longitudes, latitudes,  altitudes)
+        Tuple of three :class:`numpy:numpy.ndarray`
+        (longitudes, latitudes, altitudes).
 
     Examples
     --------
@@ -312,10 +325,10 @@ def beam_height_n(r, theta, re=6370040., ke=4. / 3.):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]
-    theta : scalar or array broadcastable to the shape of r
-        elevation angles in degrees with 0° at horizontal and +90°
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]
+    theta : scalar or :class:`numpy:numpy.ndarray` broadcastable to the shape
+        of r elevation angles in degrees with 0° at horizontal and +90°
         pointing vertically upwards from the radar
     re : float
         earth's radius [m]
@@ -351,10 +364,10 @@ def arc_distance_n(r, theta, re=6370040., ke=4. / 3.):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]
-    theta : scalar or array broadcastable to the shape of r
-        elevation angles in degrees with 0° at horizontal and +90°
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]
+    theta : scalar or :class:`numpy:numpy.ndarray` broadcastable to the shape
+        of r elevation angles in degrees with 0° at horizontal and +90°
         pointing vertically upwards from the radar
     re : float
         earth's radius [m]
@@ -388,33 +401,33 @@ def polar2lonlatalt_n(r, az, elev, sitecoords, re=None, ke=4. / 3.):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]
-    az : array
-        array of azimuth angles containing values between 0 and 360°.
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]
+    az : :class:`numpy:numpy.ndarray`
+        Array of azimuth angles containing values between 0 and 360°.
         These are assumed to start with 0° pointing north and counted
         positive clockwise!
-    th : scalar or array of the same shape as az
-        elevation angles in degrees starting with 0° at horizontal to 90°
+    th : scalar or :class:`numpy:numpy.ndarray` of the same shape as az
+        Elevation angles in degrees starting with 0° at horizontal to 90°
         pointing vertically upwards from the radar
     sitecoords : a sequence of two or three floats
-        the lon / lat coordinates of the radar location, the third value,
+        The lon / lat coordinates of the radar location, the third value,
         if present, will be interpreted as the height of the site above the
         geoid (i.e. sphere)
     re : float
-        earth's radius [m], if None, `get_earth_radius` will be used to
+        Earth's radius [m], if None, `get_earth_radius` will be used to
         determine the equivalent radius of the WGS84 ellipsoid for the
         latitude given in sitecoords.
     ke : float
-        adjustment factor to account for the refractivity gradient that
+        Adjustment factor to account for the refractivity gradient that
         affects radar beam propagation. In principle this is wavelength-
         dependent. The default of 4/3 is a good approximation for most
         weather radar wavelengths
 
     Returns
     -------
-    lon, lat, alt : tuple of arrays
-        three arrays containing the spherical longitude and latitude
+    lon, lat, alt : tuple of :class:`numpy:numpy.ndarray`
+        Three arrays containing the spherical longitude and latitude
         coordinates as well as the altitude of the beam.
 
     Note
@@ -505,16 +518,16 @@ def centroid2polyvert(centroid, delta):
     Parameters
     ----------
     centroid : array_like
-               list of 2-D coordinates of the center point of the rectangle
-    delta :    scalar or array
-               symmetric distances of the vertices from the centroid in each
+               List of 2-D coordinates of the center point of the rectangle.
+    delta :    scalar or :class:`numpy:numpy.ndarray`
+               Symmetric distances of the vertices from the centroid in each
                direction. If `delta` is scalar, it is assumed to apply to
                both dimensions.
 
     Returns
     -------
-    vertices : array
-               an array with 5 vertices per centroid.
+    vertices : :class:`numpy:numpy.ndarray`
+               An array with 5 vertices per centroid.
 
     Note
     ----
@@ -577,11 +590,11 @@ def polar2polyvert(r, az, sitecoords):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]; r defines the exterior boundaries of the range
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]; r defines the exterior boundaries of the range
         bins! (not the centroids). Thus, values must be positive!
-    az : array
-        array of azimuth angles containing values between 0° and 360°.
+    az : :class:`numpy:numpy.ndarray`
+        Array of azimuth angles containing values between 0° and 360°.
         The angles are assumed to describe the pointing direction fo the main
         beam lobe!
         The first angle can start at any values, but make sure the array is
@@ -592,8 +605,8 @@ def polar2polyvert(r, az, sitecoords):
 
     Returns
     -------
-    output : array
-        a 3-d array of polygon vertices in lon/lat with shape(num_vertices,
+    output : :class:`numpy:numpy.ndarray`
+        A 3-d array of polygon vertices in lon/lat with shape(num_vertices,
         num_vertex_nodes, 2). The last dimension carries the longitudes on
         the first position, the latitudes on the second (lon: output[:,:,0],
         lat: output[:,:,1]
@@ -604,7 +617,7 @@ def polar2polyvert(r, az, sitecoords):
     >>> import numpy as np
     >>> import matplotlib as mpl
     >>> import matplotlib.pyplot as pl
-    >>> #pl.interactive(True)
+    >>> pl.interactive(True)
     >>> # define the polar coordinates and the site coordinates in lat/lon
     >>> r = np.array([50., 100., 150., 200.])
     >>> # _check_polar_coords fails in next line
@@ -665,11 +678,11 @@ def polar2centroids(r=None, az=None, sitecoords=None, range_res=None):
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]; r defines the exterior boundaries of the range
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]; r defines the exterior boundaries of the range
         bins! (not the centroids). Thus, values must be positive!
-    az : array
-        array of azimuth angles containing values between 0° and 360°.
+    az : :class:`numpy:numpy.ndarray`
+        Array of azimuth angles containing values between 0° and 360°.
         The angles are assumed to describe the pointing direction fo the main
         beam lobe!
         The first angle can start at any values, but make sure the array is
@@ -684,8 +697,8 @@ def polar2centroids(r=None, az=None, sitecoords=None, range_res=None):
     Returns
     -------
     output : tuple
-        tuple of 2 arrays which describe the bin centroids
-        longitude and latitude
+        Tuple of 2 :class:`numpy:numpy.ndarray` which describe the bin
+        centroids longitude and latitude.
 
     Note
     ----
@@ -714,8 +727,10 @@ def _check_polar_coords(r, az):
 
     Parameters
     ----------
-    r : range gates in any unit
-    az : azimuth angles in degree
+    r : :class:`numpy:numpy.ndarray`
+        range gates in any unit
+    az : :class:`numpy:numpy.ndarray`
+        azimuth angles in degree
 
     """
     r = np.array(r, 'f4')
@@ -926,10 +941,10 @@ def projected_bincoords_from_radarspecs(r, az, sitecoords, proj,
 
     Parameters
     ----------
-    r : array
-        array of ranges [m]; r defines the exterior boundaries of the range
+    r : :class:`numpy:numpy.ndarray`
+        Array of ranges [m]; r defines the exterior boundaries of the range
         bins! (not the centroids). Thus, values must be positive!
-    az : array
+    az : :class:`numpy:numpy.ndarray`
     sitecoords : tuple
         array of azimuth angles containing values between 0° and 360°.
         The angles are assumed to describe the pointing direction fo the main
@@ -998,8 +1013,8 @@ def pixel_coordinates(nx, ny, mode="centers"):
         otherwise the pixel edges coordinates will be returned
     Returns
     -------
-    coordinates : np array
-         array of shape (ny,nx) with pixel coordinates (x,y)
+    coordinates : :class:`numpy:numpy.ndarray`
+         Array of shape (ny,nx) with pixel coordinates (x,y)
 
     """
     if mode == "centroids":
@@ -1024,7 +1039,7 @@ def pixel_to_map(geotransform, coordinates):
 
     Parameters
     ----------
-    geotransform : np array
+    geotransform : :class:`numpy:numpy.ndarray`
         geographical transformation vector:
 
             - geotransform[0] = East/West location of Upper Left corner
@@ -1033,12 +1048,12 @@ def pixel_to_map(geotransform, coordinates):
             - geotransform[3] = North/South location of Upper Left corner
             - geotransform[4] = Y pixel rotation
             - geotransform[5] = Y pixel size
-    coordinates : 2d array
-        array of pixel coordinates
+    coordinates : :class:`numpy:numpy.ndarray`
+        2d array of pixel coordinates
 
     Returns
     -------
-    coordinates_map : np array
+    coordinates_map : :class:`numpy:numpy.ndarray`
         3d array with map coordinates x,y
     """
     coordinates_map = np.empty(coordinates.shape)
@@ -1057,11 +1072,11 @@ def pixel_to_map3d(geotransform, coordinates, z=None):
 
     Parameters
     ----------
-    geotransform : np array
+    geotransform : :class:`numpy:numpy.ndarray`
         geographical transformation vector
         (see :meth:`~wradlib.georef.pixel_to_map`)
-    coordinates : 2d array
-        array of pixel coordinates;
+    coordinates : :class:`numpy:numpy.ndarray`
+        2d array of pixel coordinates;
     z : string
         method to compute the z coordinates (height above ellipsoid):
 
@@ -1070,7 +1085,7 @@ def pixel_to_map3d(geotransform, coordinates, z=None):
 
     Returns
     -------
-    coordinates_map : 4d array
+    coordinates_map : :class:`numpy:numpy.ndarray`
         4d array with map coordinates x,y,z
 
     """
@@ -1095,8 +1110,8 @@ def read_gdal_coordinates(dataset, mode='centers', z=True):
 
     Returns
     -------
-    coordinates : 3D np array
-        projected coordinates (x,y,z)
+    coordinates : :class:`numpy:numpy.ndarray`
+        Array of projected coordinates (x,y,z)
 
     Examples
     --------
@@ -1132,14 +1147,14 @@ def read_gdal_projection(dset):
     See :ref:`notebooks/classify/wradlib_clutter_cloud_example.ipynb`.
 
     """
-    proj4 = dset.GetProjection()
+    wkt = dset.GetProjection()
     srs = osr.SpatialReference()
-    srs.ImportFromProj4(proj4)
+    srs.ImportFromWkt(wkt)
     # src = None
     return srs
 
 
-def read_gdal_values(data=None, nodata=False):
+def read_gdal_values(dataset=None, nodata=None):
     """Read values from a gdal object.
 
     Parameters
@@ -1150,8 +1165,9 @@ def read_gdal_values(data=None, nodata=False):
 
     Returns
     -------
-    values : 2d array
-        array with values
+    values : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols) or (bands, rows, cols) containing
+        the data values.
 
     Examples
     --------
@@ -1159,14 +1175,19 @@ def read_gdal_values(data=None, nodata=False):
     See :ref:`notebooks/classify/wradlib_clutter_cloud_example.ipynb`.
 
     """
+    nbands = dataset.RasterCount
 
-    b1 = data.GetRasterBand(1)
-    values = b1.ReadAsArray()
-    if nodata:
-        nodata = b1.GetNoDataValue()
-        values = values.astype('float')
-        values[values == nodata] = np.nan
-    return (values)
+    # data values
+    bands = []
+    for i in range(nbands):
+        band = dataset.GetRasterBand(i + 1)
+        nd = band.GetNoDataValue()
+        data = band.ReadAsArray()
+        if nodata is not None:
+            data[data == nd] = nodata
+        bands.append(data)
+
+    return np.squeeze(np.stack(bands))
 
 
 def reproject(*args, **kwargs):
@@ -1183,15 +1204,15 @@ def reproject(*args, **kwargs):
 
     Parameters
     ----------
-    C : multidimensional np array
-        array of shape (...,2) or (...,3) with coordinates (x,y) or (x,y,z)
+    C : multidimensional :class:`numpy:numpy.ndarray`
+        Array of shape (...,2) or (...,3) with coordinates (x,y) or (x,y,z)
         respectively
-    X : nd array
-        array of x coordinates
-    Y : nd array
-        array of y coordinates
-    Z : nd array
-        array of z coordinates
+    X : :class:`numpy:numpy.ndarray`
+        Array of x coordinates
+    Y : :class:`numpy:numpy.ndarray`
+        Array of y coordinates
+    Z : :class:`numpy:numpy.ndarray`
+        Array of z coordinates
 
     Keyword Arguments
     -----------------
@@ -1202,13 +1223,13 @@ def reproject(*args, **kwargs):
 
     Returns
     -------
-    trans : nd array
-        array of reprojected coordinates x,y (...,2) or x,y,z (...,3)
-        depending on input array
-    X, Y : nd arrays
-        arrays of reprojected x,y coordinates, shape depending on input array
-    X, Y, Z: nd arrays
-        arrays of reprojected x,y,z coordinates, shape depending on input array
+    trans : :class:`numpy:numpy.ndarray`
+        Array of reprojected coordinates x,y (...,2) or x,y,z (...,3)
+        depending on input array.
+    X, Y : :class:`numpy:numpy.ndarray`
+        Arrays of reprojected x,y coordinates, shape depending on input array
+    X, Y, Z: :class:`numpy:numpy.ndarray`
+        Arrays of reprojected x,y,z coordinates, shape depending on input array
 
     Examples
     --------
@@ -1370,9 +1391,9 @@ def get_radolan_coords(lon, lat, trig=False):
     Parameters
     ----------
 
-    lon :   float, array of floats
+    lon :   float, :class:`numpy:numpy.ndarray` of floats
         longitude
-    lat :   float, array of floats
+    lat :   float, :class:`numpy:numpy.ndarray` of floats
         latitude
     trig : boolean
         if True, uses trigonometric formulas for calculation,
@@ -1433,6 +1454,20 @@ def get_radolan_grid(nrows=None, ncols=None, trig=False, wgs84=False):
         | UpperLeft  |  2.0715E  |  54.5877N  | -523.4622 | -3758.645 |
         +------------+-----------+------------+-----------+-----------+
 
+    .. table:: Coordinates for 1100km x 900km grid
+
+        +------------+-----------+------------+-----------+-----------+
+        | Coordinate |   lon     |     lat    |     x     |     y     |
+        +============+===========+============+===========+===========+
+        | LowerLeft  |  4.6759E  |  46.1929N  | -443.4622 | -4758.645 |
+        +------------+-----------+------------+-----------+-----------+
+        | LowerRight | 15.4801E  |  46.1827N  |  456.5378 | -4758.645 |
+        +------------+-----------+------------+-----------+-----------+
+        | UpperRight | 17.1128E  |  55.5342N  |  456.5378 | -3658.645 |
+        +------------+-----------+------------+-----------+-----------+
+        | UpperLeft  |  3.0889E  |  55.5482N  | -433.4622 | -3658.645 |
+        +------------+-----------+------------+-----------+-----------+
+
     .. table:: Coordinates for 1500km x 1400km grid
 
         +------------+-----------+------------+-----------+-----------+
@@ -1444,7 +1479,7 @@ def get_radolan_grid(nrows=None, ncols=None, trig=False, wgs84=False):
     Parameters
     ----------
     nrows : int
-        number of rows (460, 900 by default, 1500)
+        number of rows (460, 900 by default, 1100, 1500)
     ncols : int
         number of columns (460, 900 by default, 1400)
     trig : boolean
@@ -1458,38 +1493,37 @@ def get_radolan_grid(nrows=None, ncols=None, trig=False, wgs84=False):
 
     Returns
     -------
-    radolan_grid : numpy ndarray (rows, cols, 2)
-                   xy- or lonlat-grid
+    radolan_grid : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols, 2) xy- or lonlat-grid.
 
     Examples
     --------
 
-        >>> # using osr spatial reference transformation
-        >>> import wradlib.georef as georef  # noqa
-        >>> radolan_grid = georef.get_radolan_grid()
-        >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
-        (900, 900, 2), (-523.4622, -4658.6447)
+    >>> # using osr spatial reference transformation
+    >>> import wradlib.georef as georef  # noqa
+    >>> radolan_grid = georef.get_radolan_grid()
+    >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
+    (900, 900, 2), (-523.4622, -4658.6447)
 
-        >>> # using pure trigonometric transformations
-        >>> import wradlib.georef as georef
-        >>> radolan_grid = georef.get_radolan_grid(trig=True)
-        >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
-        (900, 900, 2), (-523.4622, -4658.6447)
+    >>> # using pure trigonometric transformations
+    >>> import wradlib.georef as georef
+    >>> radolan_grid = georef.get_radolan_grid(trig=True)
+    >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
+    (900, 900, 2), (-523.4622, -4658.6447)
 
-        >>> # using osr spatial reference transformation
-        >>> import wradlib.georef as georef
-        >>> radolan_grid = georef.get_radolan_grid(1500, 1400)
-        >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
-        (1500, 1400, 2), (-673.4622, -5008.6447)
+    >>> # using osr spatial reference transformation
+    >>> import wradlib.georef as georef
+    >>> radolan_grid = georef.get_radolan_grid(1500, 1400)
+    >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
+    (1500, 1400, 2), (-673.4622, -5008.6447)
 
-        >>> # using osr spatial reference transformation
-        >>> import wradlib.georef as georef
-        >>> radolan_grid = georef.get_radolan_grid(900, 900, wgs84=True)
-        >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
-        (900, 900, 2), (3.5889, 46.9526)
+    >>> # using osr spatial reference transformation
+    >>> import wradlib.georef as georef
+    >>> radolan_grid = georef.get_radolan_grid(900, 900, wgs84=True)
+    >>> print("{0}, ({1:.4f}, {2:.4f})".format(radolan_grid.shape, *radolan_grid[0,0,:]))  # noqa
+    (900, 900, 2), (3.5889, 46.9526)
 
-    See :ref:`notebooks/radolan/radolan_grid.ipynb#\
-Polar-Stereographic-Projection`.
+    See :ref:`notebooks/radolan/radolan_grid.ipynb#Polar-Stereographic-Projection`.  # noqa
 
     Raises
     ------
@@ -1500,9 +1534,11 @@ Polar-Stereographic-Projection`.
     tiny = {'j_0': 450, 'i_0': 450, 'res': 2}
     small = {'j_0': 460, 'i_0': 460, 'res': 2}
     normal = {'j_0': 450, 'i_0': 450, 'res': 1}
+    normal_wx = {'j_0': 370, 'i_0': 550, 'res': 1}
     extended = {'j_0': 600, 'i_0': 800, 'res': 1}
     griddefs = {(450, 450): tiny, (460, 460): small,
-                (900, 900): normal, (1500, 1400): extended}
+                (900, 900): normal, (1100, 900): normal_wx,
+                (1500, 1400): extended}
 
     # type and value checking
     if nrows and ncols:
@@ -1560,6 +1596,140 @@ Polar-Stereographic-Projection`.
     return radolan_grid
 
 
+def reproject_raster_dataset(src_ds, **kwargs):
+    """Reproject/Resample given dataset according to keyword arguments
+
+    .. versionadded:: 0.10.0
+
+    # function inspired from github project
+    # https://github.com/profLewis/geogg122
+
+    Parameters
+    ----------
+    src_ds : gdal.Dataset
+        raster image with georeferencing (GeoTransform at least)
+    spacing : float
+        float or tuple of two floats
+        pixel spacing of destination dataset, same unit as pixel coordinates
+    size : int
+        tuple of two ints
+        X/YRasterSize of destination dataset
+    resample : GDALResampleAlg
+        defaults to GRA_Bilinear
+        GRA_NearestNeighbour = 0, GRA_Bilinear = 1, GRA_Cubic = 2,
+        GRA_CubicSpline = 3, GRA_Lanczos = 4, GRA_Average = 5, GRA_Mode = 6,
+        GRA_Max = 8, GRA_Min = 9, GRA_Med = 10, GRA_Q1 = 11, GRA_Q3 = 12
+    projection_target : osr object
+        destination dataset projection, defaults to None
+    align : bool or Point
+        If False, there is no destination grid aligment.
+        If True, aligns the destination grid to the next integer multiple of
+        destination grid.
+        If Point (tuple, list of upper-left x,y-coordinate), the destination
+        grid is aligned to this point.
+
+    Returns
+    -------
+    dst_ds : gdal.Dataset
+        reprojected/resampled raster dataset
+    """
+
+    # checking kwargs
+    spacing = kwargs.pop('spacing', None)
+    size = kwargs.pop('size', None)
+    resample = kwargs.pop('resample', gdal.GRA_Bilinear)
+    src_srs = kwargs.pop('projection_source', None)
+    dst_srs = kwargs.pop('projection_target', None)
+    align = kwargs.pop('align', False)
+
+    # Get the GeoTransform vector
+    src_geo = src_ds.GetGeoTransform()
+    x_size = src_ds.RasterXSize
+    y_size = src_ds.RasterYSize
+
+    # get extent
+    ulx = src_geo[0]
+    uly = src_geo[3]
+    lrx = src_geo[0] + src_geo[1] * x_size
+    lry = src_geo[3] + src_geo[5] * y_size
+
+    extent = np.array([[[ulx, uly],
+                        [lrx, uly]],
+                       [[ulx, lry],
+                        [lrx, lry]]])
+
+    if dst_srs:
+        print("dest_src available")
+        src_srs = osr.SpatialReference()
+        src_srs.ImportFromWkt(src_ds.GetProjection())
+
+        # Transformation
+        extent = reproject(extent, projection_source=src_srs,
+                           projection_target=dst_srs)
+
+        # wkt needed
+        src_srs = src_srs.ExportToWkt()
+        dst_srs = dst_srs.ExportToWkt()
+
+    (ulx, uly, urx, ury,
+     llx, lly, lrx, lry) = tuple(list(extent.flatten().tolist()))
+
+    # align grid to destination raster or UL-corner point
+    if align:
+        try:
+            ulx, uly = align
+        except TypeError:
+            pass
+
+        ulx = int(max(np.floor(ulx), np.floor(llx)))
+        uly = int(min(np.ceil(uly), np.ceil(ury)))
+        lrx = int(min(np.ceil(lrx), np.ceil(urx)))
+        lry = int(max(np.floor(lry), np.floor(lly)))
+
+    # calculate cols/rows or xspacing/yspacing
+    if spacing:
+        try:
+            x_ps, y_ps = spacing
+        except TypeError:
+            x_ps = spacing
+            y_ps = spacing
+
+        cols = int(abs(lrx - ulx) / x_ps)
+        rows = int(abs(uly - lry) / y_ps)
+    elif size:
+        cols, rows = size
+        x_ps = x_size * src_geo[1] / cols
+        y_ps = y_size * abs(src_geo[5]) / rows
+    else:
+        raise NameError("Whether keyword 'spacing' or 'size' must be given")
+
+    # create destination in-memory raster
+    mem_drv = gdal.GetDriverByName('MEM')
+
+    # and set RasterSize according ro cols/rows
+    dst_ds = mem_drv.Create('', cols, rows, 1, gdal.GDT_Float32)
+
+    # Create the destination GeoTransform with changed x/y spacing
+    dst_geo = (ulx, x_ps, src_geo[2], uly, src_geo[4], -y_ps)
+
+    # apply GeoTransform to destination dataset
+    dst_ds.SetGeoTransform(dst_geo)
+
+    # nodata handling, need to initialize dst_ds with nodata
+    src_band = src_ds.GetRasterBand(1)
+    nodata = src_band.GetNoDataValue()
+    dst_band = dst_ds.GetRasterBand(1)
+    dst_band.SetNoDataValue(nodata)
+    dst_band.WriteArray(np.ones((rows, cols)) * nodata)
+    dst_band.FlushCache()
+
+    # resample and reproject dataset
+    gdal.ReprojectImage(src_ds, dst_ds, src_srs, dst_srs, resample)
+
+    return dst_ds
+
+
+@util.deprecated(reproject_raster_dataset)
 def resample_raster_dataset(src_ds, **kwargs):
     """Resample given dataset according to keyword arguments
 
@@ -1619,7 +1789,7 @@ def resample_raster_dataset(src_ds, **kwargs):
 
     # create destination in-memory raster
     mem_drv = gdal.GetDriverByName('MEM')
-    # and set RasterSize according ro cols/rows
+    # and set RasterSize according cols/rows
     dst_ds = mem_drv.Create('', cols, rows, 1, gdal.GDT_Float32)
 
     # Create the destination GeoTransform with changed x/y spacing
@@ -1689,11 +1859,8 @@ def transform_geometry(geom, dest_srs):
         Transformed Geometry
     """
 
-    src_srs = geom.GetSpatialReference()
-    # needed to get EPSG code
-    src_srs.AutoIdentifyEPSG()
     # transform if not the same spatial reference system
-    if src_srs.GetAuthorityCode(None) != dest_srs.GetAuthorityCode(None):
+    if not geom.GetSpatialReference().IsSame(dest_srs):
         geom.TransformTo(dest_srs)
 
     return geom
@@ -1722,7 +1889,7 @@ def get_shape_coordinates(layer, **kwargs):
 
     Returns
     -------
-    shp : nested ndarray
+    shp : nested :class:`numpy:numpy.ndarray`
         Dimension of subarrays Nx2
         extracted shape coordinate points
     attrs : list
@@ -1755,6 +1922,284 @@ def get_shape_coordinates(layer, **kwargs):
     return shp, attrs
 
 
+def correct_parallax(pr_xy, nbin, drt, alpha):
+    """Adjust the geo-locations of the PR pixels
+
+    With *PR*, we refer to precipitation radars based on space-born platforms
+    such as TRMM or GPM.
+
+    The `pr_xy` coordinates of the PR beam footprints need to be in the
+    azimuthal equidistant projection of the ground radar. This ensures that the
+    ground radar is fixed at xy-coordinate (0, 0), and every PR bin has its
+    relative xy-coordinates with respect to the ground radar site.
+
+    .. versionadded:: 0.10.0
+
+    Parameters
+    ----------
+    pr_xy : :class:`numpy:numpy.ndarray`
+        Array of xy-coordinates of shape (nscans, nbeams, 2)
+    nbin : int
+        Number of bins along PR beam.
+    drt : float
+        Gate lenght of PR in meter.
+    alpha: :class:`numpy:numpy.ndarray`
+        Array of depression angles of the PR beams with shape (nbeams).
+
+    Returns
+    -------
+
+    pr_xyp : :class:`numpy:numpy.ndarray`
+        Array of parallax corrected coordinates
+        of shape (nscans, nbeams, nbins, 2).
+    r_pr_inv : :class:`numpy:numpy.ndarray`
+        Array of ranges from ground to PR platform of shape (nbins).
+    z_pr : :class:`numpy:numpy.ndarray`
+        Array of PR bin altitudes of shape (nbeams, nbins).
+    """
+    # get x,y-grids
+    pr_x = pr_xy[..., 0]
+    pr_y = pr_xy[..., 1]
+
+    # create range array from ground to sat
+    r_pr_inv = np.arange(nbin) * drt
+
+    # calculate height of bin
+    z_pr = r_pr_inv * np.cos(np.deg2rad(alpha))[:, np.newaxis]
+    # calculate bin ground xy-displacement length
+    ds = r_pr_inv * np.sin(np.deg2rad(alpha))[:, np.newaxis]
+
+    # calculate x,y-differences between ground coordinate
+    # and center ground coordinate [25th element]
+    center = int(np.floor(len(pr_x[-1]) / 2.))
+    xdiff = pr_x[:, center][:, np.newaxis] - pr_x
+    ydiff = pr_y[:, center][:, np.newaxis] - pr_y
+
+    # assuming ydiff and xdiff being a triangles adjacent and
+    # opposite this calculates the xy-angle of the PR scan
+    ang = np.arctan2(ydiff, xdiff)
+
+    # calculate displacement dx, dy from displacement length
+    dx = ds * np.cos(ang)[..., np.newaxis]
+    dy = ds * np.sin(ang)[..., np.newaxis]
+
+    # add displacement to PR ground coordinates
+    pr_xp = dx + pr_x[..., np.newaxis]
+    pr_yp = dy + pr_y[..., np.newaxis]
+
+    return np.stack((pr_xp, pr_yp), axis=3), r_pr_inv, z_pr
+
+
+def sat2pol(pr_xyz, gr_site_alt, re):
+    """Returns spherical coordinates of PR bins as seen from the GR location.
+
+    With *PR*, we refer to precipitation radars based on space-born platforms
+    such as TRMM or GPM. With *GR*, we refer to terrestrial weather radars
+    (ground radars).
+
+    For this function to work, the `pr_xyz` coordinates of the PR bins need
+    to be in the azimuthal equidistant projection of the ground radar! This
+    ensures that the ground radar is fixed at xy-coordinate (0, 0), and every
+    PR bin has its relative xy-coordinates with respect to the ground radar
+    site.
+
+    .. versionadded:: 0.10.0
+
+    Parameters
+    ----------
+    pr_xyz : :class:`numpy:numpy.ndarray`
+        Array of shape (nscans, nbeams, nbins, 3). Contains corrected
+        PR xy coordinates in GR azimuthal equidistant projection and altitude
+    gr_site_alt : float
+        Altitude of the GR site (in meters)
+    re : float
+        Effective Earth radius at GR site (in meters)
+
+    Returns
+    -------
+    r : :class:`numpy:numpy.ndarray`
+        Array of shape (nscans, nbeams, nbins). Contains the slant
+        distance of PR bins from GR site.
+    theta: :class:`numpy:numpy.ndarray`
+        Array of shape (nscans, nbeams, nbins). Contains the elevation
+        angle of PR bins as seen from GR site.
+    phi : :class:`numpy:numpy.ndarray`
+        Array of shape (nscans, nbeams, nbins). Contains the azimuth
+        angles of PR bins as seen from GR site.
+    """
+    # calculate arc length
+    s = np.sqrt(np.sum(pr_xyz[..., 0:2] ** 2, axis=-1))
+
+    # calculate arc angle
+    gamma = s / re
+
+    # calculate theta (elevation-angle)
+    numer = np.cos(gamma) - (re + gr_site_alt) / (re + pr_xyz[..., 2])
+    denom = np.sin(gamma)
+    theta = np.rad2deg(np.arctan(numer / denom))
+
+    # calculate SlantRange r
+    r = (re + pr_xyz[..., 2]) * denom / np.cos(np.deg2rad(theta))
+
+    # calculate Azimuth phi
+    phi = 90 - np.rad2deg(np.arctan2(pr_xyz[..., 1], pr_xyz[..., 0]))
+    phi[phi <= 0] += 360
+
+    return r, theta, phi
+
+
+def dist_from_orbit(pr_alt, alpha, r_pr_inv):
+    """Returns range distances of PR bins (in meters) as seen from the orbit
+
+    With *PR*, we refer to precipitation radars based on space-born platforms
+    such as TRMM or GPM.
+
+    .. versionadded:: 0.10.0
+
+    Parameters
+    ----------
+    pr_alt : float
+        PR orbit height in meters.
+    alpha: :class:`numpy:numpy.ndarray`
+       Array of depression angles of the PR beams with shape (nbeams).
+    r_pr_inv : :class:`numpy:numpy.ndarray`
+        Array of ranges from ground to PR platform of shape (nbins).
+
+    Returns
+    -------
+    ranges : :class:`numpy:numpy.ndarray`
+        Array of shape (nbeams, nbins) of PR bin range distances from
+        PR platform in orbit.
+    """
+    return pr_alt / np.cos(np.radians(alpha))[:, np.newaxis] - r_pr_inv
+
+
+def create_raster_dataset(data, coords, projection=None, nodata=-9999):
+    """ Create In-Memory Raster Dataset
+
+    .. versionadded 0.10.0
+
+    Parameters
+    ----------
+    data : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols) or (bands, rows, cols) containing
+        the data values.
+    coords : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols, 2) containing xy-coordinates.
+    projection : osr object
+        Spatial reference system of the used coordinates, defaults to None.
+
+    Returns
+    -------
+    dataset : gdal.Dataset
+        In-Memory raster dataset
+
+    Note
+    ----
+    The origin of the provided data and coordinates is UPPER LEFT.
+    """
+
+    # align data
+    data = data.copy()
+    if data.ndim == 2:
+        data = data[np.newaxis, ...]
+    bands, rows, cols = data.shape
+
+    # create In-Memory Raster with correct dtype
+    mem_drv = gdal.GetDriverByName('MEM')
+    gdal_type = gdal_array.NumericTypeCodeToGDALTypeCode(data.dtype)
+    dataset = mem_drv.Create('', cols, rows, bands, gdal_type)
+
+    # initialize geotransform
+    x_ps, y_ps = coords[1, 1] - coords[0, 0]
+    geotran = [coords[0, 0, 0], x_ps, 0, coords[0, 0, 1], 0, y_ps]
+    dataset.SetGeoTransform(geotran)
+
+    if projection:
+        dataset.SetProjection(projection.ExportToWkt())
+
+    # set np.nan to nodata
+    dataset.GetRasterBand(1).SetNoDataValue(nodata)
+
+    for i, band in enumerate(data, start=1):
+        dataset.GetRasterBand(i).WriteArray(band)
+
+    return dataset
+
+
+def set_raster_origin(data, coords, direction):
+    """ Converts Data and Coordinates Origin
+
+    .. versionadded 0.10.0
+
+    Parameters
+    ----------
+    data : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols) or (bands, rows, cols) containing
+        the data values.
+    coords : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols, 2) containing xy-coordinates.
+    direction : str
+        'lower' or 'upper', direction in which to convert data and coordinates.
+
+    Returns
+    -------
+    data : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols) or (bands, rows, cols) containing
+        the data values.
+    coords : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols, 2) containing xy-coordinates.
+    """
+    x_sp, y_sp = coords[1, 1] - coords[0, 0]
+    origin = ('lower' if y_sp > 0 else 'upper')
+    same = (origin == direction)
+    if not same:
+        data = np.flip(data, axis=-2)
+        coords = np.flip(coords, axis=-3)
+        # we need to shift y-coordinate if data and coordinates have the same
+        # number of rows and cols
+        if data.shape[-2:] == coords.shape[:2]:
+            coords += [0, y_sp]
+
+    return data, coords
+
+
+def extract_raster_dataset(dataset, nodata=None):
+    """ Extract data, coordinates and projection information
+
+    Parameters
+    ----------
+    dataset : gdal.Dataset
+        raster dataset
+    nodata : scalar
+        Value to which the dataset nodata values are mapped.
+
+    Returns
+    -------
+    data : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols) or (bands, rows, cols) containing
+        the data values.
+    coords : :class:`numpy:numpy.ndarray`
+        Array of shape (rows, cols, 2) containing xy-coordinates.
+    projection : osr object
+        Spatial reference system of the used coordinates.
+    """
+
+    # data values
+    data = read_gdal_values(dataset, nodata=nodata)
+
+    # coords
+    coords_pixel = pixel_coordinates(dataset.RasterXSize,
+                                     dataset.RasterYSize,
+                                     'edges')
+    coords = pixel_to_map(dataset.GetGeoTransform(),
+                          coords_pixel)
+
+    projection = read_gdal_projection(dataset)
+
+    return data, coords, projection
+
+
 def _doctest_():
     import doctest
     print('doctesting')
