@@ -481,7 +481,7 @@ def get_radolan_header_token():
     head = {'BY': None, 'VS': None, 'SW': None, 'PR': None,
             'INT': None, 'GP': None, 'MS': None, 'LV': None,
             'CS': None, 'MX': None, 'BG': None, 'ST': None,
-            'VV': None, 'MF': None}
+            'VV': None, 'MF': None, 'QN': None}
     return head
 
 
@@ -600,6 +600,8 @@ def parse_DWD_quant_composite_header(header):
                 out['predictiontime'] = int(header[v[0]:v[1]])
             if k == 'MF':
                 out['moduleflag'] = int(header[v[0]:v[1]])
+            if k == 'QN':
+                out['quantification'] = int(header[v[0]:v[1]])
     return out
 
 
@@ -1998,7 +2000,7 @@ def write_raster_dataset(fpath, dataset, format, options=None, remove=False):
     options : list
         List of option strings for the corresponding format.
     remove : bool
-        if True, existing OGR.DataSource will be
+        if True, existing gdal.Dataset will be
         removed before creation
 
     Note
@@ -2334,7 +2336,7 @@ def read_raster_data(filename, driver=None, **kwargs):
 
 def open_shape(filename, driver=None):
     """
-    Open shapefile, return ogr dataset and layer
+    Open shapefile, return gdal.Dataset and OGR.Layer
 
     .. warning:: dataset and layer have to live in the same context,
                  if dataset is deleted all layer references will get lost
@@ -2350,7 +2352,7 @@ def open_shape(filename, driver=None):
 
     Returns
     -------
-    dataset : ogr.Dataset
+    dataset : gdal.Dataset
         dataset
     layer : ogr.Layer
         layer
@@ -2368,7 +2370,7 @@ def open_shape(filename, driver=None):
 
 def open_raster(filename, driver=None):
     """
-    Open raster file, return ogr dataset
+    Open raster file, return gdal.Dataset
 
     .. versionadded:: 0.6.0
 
@@ -2381,7 +2383,7 @@ def open_raster(filename, driver=None):
 
     Returns
     -------
-    dataset : ogr.Dataset
+    dataset : gdal.Dataset
         dataset
     """
 
