@@ -46,9 +46,15 @@ Calling the objects with actual data, however, will be very fast.
    ZonalStatsBase
    GridCellsToPoly
    GridPointsToPoly
+   mask_from_bbox
+   get_bbox
+   grid_centers_to_vertices
+   get_clip_mask
 
 """
 
+from . import util as util
+
 import os
 import numpy as np
 from scipy.spatial import cKDTree
@@ -185,7 +191,8 @@ DataSource`.
             - transforming source grid points/polygons to ogr.geometries
               on ogr.layer
         """
-        ogr_src = ogr_create_datasource('Memory', 'out')
+        ogr_src = gdal_create_dataset('Memory', 'out',
+                                      gdal_type=gdal.OF_VECTOR)
 
         try:
             # is it ESRI Shapefile?
@@ -226,7 +233,8 @@ DataSource`.
             if True removes existing output file
 
         """
-        ds_out = ogr_create_datasource(driver, filename, remove=remove)
+        ds_out = gdal_create_dataset(driver, filename,
+                                     gdal_type=gdal.OF_VECTOR, remove=remove)
         ogr_copy_layer(self.ds, 0, ds_out)
 
         # flush everything
@@ -259,8 +267,9 @@ DataSource`.
         rows = int((y_max - y_min) / pixel_size)
 
         # Todo: at the moment, always writing floats
-        ds_out = gdal_create_dataset(driver, filename, cols, rows,
-                                     gdal.GDT_Float32, remove=remove)
+        ds_out = gdal_create_dataset('MEM', '', cols, rows, 1,
+                                     gdal_type=gdal.GDT_Float32)
+
         ds_out.SetGeoTransform((x_min, pixel_size, 0, y_max, 0, -pixel_size))
         proj = layer.GetSpatialRef()
         if proj is None:
@@ -269,7 +278,7 @@ DataSource`.
 
         band = ds_out.GetRasterBand(1)
         band.FlushCache()
-        print("Rasterizing Layer")
+        print("Rasterize layers")
         if attr is not None:
             gdal.RasterizeLayer(ds_out, [1], layer, burn_values=[0],
                                 options=["ATTRIBUTE={0}".format(attr),
@@ -280,6 +289,8 @@ DataSource`.
                                 options=["ALL_TOUCHED=TRUE"],
                                 callback=gdal.TermProgress)
 
+        io.write_raster_dataset(filename, ds_out, driver, remove=remove)
+
         del ds_out
 
     def set_attribute(self, name, values):
@@ -458,20 +469,20 @@ class ZonalDataBase(object):
                                                 filt=('trg_index', idx))[0])
 
     def _create_dst_datasource(self, **kwargs):
-        """ Create destination target OGR.DataSource
+        """ Create destination target gdal.Dataset
 
         Creates one layer for each target polygon, consisting of
         the needed source data attributed with index and weights fields
 
         Returns
         -------
-        ds_mem : OGR.DataSource object
+        ds_mem : gdal.Dataset object
         """
 
         # TODO: kwargs necessary?
 
-        # create intermediate mem datasource
-        ds_mem = ogr_create_datasource('Memory', 'dst')
+        # create intermediate mem dataset
+        ds_mem = gdal_create_dataset('Memory', 'dst', gdal_type=gdal.OF_VECTOR)
 
         # get src geometry layer
         src_lyr = self.src.ds.GetLayerByName('src')
@@ -480,7 +491,7 @@ class ZonalDataBase(object):
         geom_type = src_lyr.GetGeomType()
 
         # create temp Buffer layer (time consuming)
-        ds_tmp = ogr_create_datasource('Memory', 'tmp')
+        ds_tmp = gdal_create_dataset('Memory', 'tmp', gdal_type=gdal.OF_VECTOR)
         ogr_copy_layer(self.trg.ds, 0, ds_tmp)
         tmp_trg_lyr = ds_tmp.GetLayer()
 
@@ -1019,11 +1030,16 @@ class GridPointsToPoly(ZonalStatsBase):
         super(GridPointsToPoly, self).__init__(src, **kwargs)
 
 
-def gdal_create_dataset(drv, name, cols, rows, gdal_type, remove=False):
+def gdal_create_dataset(drv, name, cols=0, rows=0, bands=0,
+                        gdal_type=gdal.GDT_Unknown, remove=False):
     """Creates GDAL.DataSet object.
 
     .. versionadded:: 0.7.0
 
+    .. versionchanged:: 0.11.0
+        - changed parameters to keyword args
+        - added 'bands' as parameter
+
     Parameters
     ----------
     drv : string
@@ -1034,15 +1050,17 @@ def gdal_create_dataset(drv, name, cols, rows, gdal_type, remove=False):
         # of columns
     rows : int
         # of rows
+    bands : int
+        # of raster bands
     gdal_type : raster data type
         eg. gdal.GDT_Float32
     remove : bool
-        if True, existing OGR.DataSource will be
+        if True, existing gdal.Dataset will be
         removed before creation
 
     Returns
     -------
-    out : OGR.DataSource
+    out : gdal.Dataset
         object
 
     """
@@ -1055,16 +1073,20 @@ def gdal_create_dataset(drv, name, cols, rows, gdal_type, remove=False):
     if remove:
         if os.path.exists(name):
             driver.Delete(name)
-    ds = driver.Create(name, cols, rows, 1, gdal_type)
+    ds = driver.Create(name, cols, rows, bands, gdal_type)
 
     return ds
 
 
+@util.deprecated(gdal_create_dataset)
 def ogr_create_datasource(drv, name, remove=False):
     """Creates OGR.DataSource object.
 
     .. versionadded:: 0.7.0
 
+    .. deprecated:: 0.12.0
+        Use :func:`gdal_create_dataset` instead.
+
     Parameters
     ----------
     drv : string
@@ -1091,16 +1113,16 @@ def ogr_create_datasource(drv, name, remove=False):
 
 
 def ogr_create_layer(ds, name, srs=None, geom_type=None, fields=None):
-    """Creates OGR.Layer objects in OGR.DataSource object.
+    """Creates OGR.Layer objects in gdal.Dataset object.
 
     .. versionadded:: 0.7.0
 
-    Creates one OGR.Layer with given name in given OGR.DataSource object
+    Creates one OGR.Layer with given name in given gdal.Dataset object
     using given OGR.GeometryType and FieldDefinitions
 
     Parameters
     ----------
-    ds : OGR.DataSource
+    ds : gdal.Dataset
         object
     name : string
         OGRLayer name
@@ -1132,15 +1154,15 @@ def ogr_copy_layer(src_ds, index, dst_ds, reset=True):
 
     .. versionadded:: 0.7.0
 
-    Copy OGR.Layer object from src_ds OGR.DataSource to dst_ds OGR.DataSource
+    Copy OGR.Layer object from src_ds gdal.Dataset to dst_ds gdal.Dataset
 
     Parameters
     ----------
-    src_ds : OGR.DataSource
+    src_ds : gdal.Dataset
         object
     index : int
         layer index
-    dst_ds : OGR.DataSource
+    dst_ds : gdal.Dataset
         object
     reset : bool
         if True resets src_layer
@@ -1160,15 +1182,15 @@ def ogr_copy_layer_by_name(src_ds, name, dst_ds, reset=True):
 
     .. versionadded:: 0.8.0
 
-    Copy OGR.Layer object from src_ds OGR.DataSource to dst_ds OGR.DataSource
+    Copy OGR.Layer object from src_ds gdal.Dataset to dst_ds gdal.Dataset
 
     Parameters
     ----------
-    src_ds : OGR.DataSource
+    src_ds : gdal.Dataset
         object
     name : string
         layer name
-    dst_ds : OGR.DataSource
+    dst_ds : gdal.Dataset
         object
     reset : bool
         if True resets src_layer
@@ -1194,7 +1216,7 @@ def ogr_add_feature(ds, src, name=None):
 
     Parameters
     ----------
-    ds : OGR.DataSource
+    ds : gdal.Dataset
         object
     src : :func:`numpy:numpy.array`
         source data
@@ -1271,8 +1293,14 @@ def numpy_to_ogr(vert, geom_name):
     """
 
     if geom_name in ['Polygon', 'MultiPolygon']:
-        json_str = "{{'type':{0!r},'coordinates':[{1!r}]}}".\
-            format(geom_name, vert.tolist())
+        # if vert is nested ndarray (only 1dim),
+        # we need to handle every item separately
+        if vert.ndim == 1:
+            lst = [item.tolist() for item in vert]
+        else:
+            lst = [vert.tolist()]
+        json_str = "{{'type':{0!r},'coordinates':{1!r}}}".\
+            format(geom_name, lst)
     else:
         json_str = "{{'type':{0!r},'coordinates':{1!r}}}".\
             format(geom_name, vert.tolist())
@@ -1404,7 +1432,7 @@ def mask_from_bbox(x, y, bbox, polar=False):
     jll = (ixll % nx) - 1
     # find upper right corner index
     dists, ixur = tree.query([bbox["right"], bbox["top"]], k=1)
-    iur = (ixur / nx) + 1
+    iur = int(ixur / nx) + 1
     jur = (ixur % nx) + 1
 
     # for polar grids we need all 4 corners
@@ -1476,9 +1504,9 @@ def get_bbox(x, y):
     Parameters
     ----------
 
-    x : array
+    x : :class:`numpy:numpy.ndarray`
         x-coordinate values
-    y : array
+    y : :class:`numpy:numpy.ndarray`
         y-coordinate values
 
     """
@@ -1517,10 +1545,10 @@ def grid_centers_to_vertices(x, y, dx, dy):
 
     Parameters
     ----------
-    x : 2-d array
-        of x coordinates (same shape as the actual 2-D grid)
-    y : 2-d array
-        of y coordinates (same shape as the actual 2-D grid)
+    x : :class:`numpy:numpy.ndarray`
+        2-d array of x coordinates (same shape as the actual 2-D grid)
+    y : :class:`numpy:numpy.ndarray`
+        2-d array of y coordinates (same shape as the actual 2-D grid)
     dx : grid spacing in x direction
     dy : grid spacing in y direction
 
@@ -1543,5 +1571,45 @@ def grid_centers_to_vertices(x, y, dx, dy):
     return verts
 
 
+def get_clip_mask(coords, clippoly, srs):
+    """Returns boolean mask of points (coords) located inside polygon clippoly
+
+    .. versionadded:: 0.10.0
+
+    Parameters
+    ----------
+    coords : :class:`numpy:numpy.ndarray`
+        array of xy coords with shape [...,2]
+
+    clippoly : :class:`numpy:numpy.ndarray`
+        array of xy coords with shape (N,2) representing closed
+        polygon coordinates
+
+    srs: osr.SpatialReference
+
+    Returns
+    -------
+    src_mask : :class:`numpy:numpy.ndarray`
+        boolean array of shape coords.shape[0:-1]
+
+    """
+    clip = [clippoly]
+
+    zd = ZonalDataPoint(coords.reshape(-1, coords.shape[-1]),
+                        clip, srs=srs)
+    obj = GridPointsToPoly(zd)
+
+    #    Get source indices within polygon from zonal object
+    #    (0 because we have only one zone)
+    pr_idx = obj.zdata.get_source_index(0)
+
+    # Subsetting in order to use only precipitating profiles
+    src_mask = np.zeros(coords.shape[0:-1], dtype=np.bool)
+    mask = np.unravel_index(pr_idx, coords.shape[0:-1])
+    src_mask[mask] = True
+
+    return src_mask
+
+
 if __name__ == '__main__':
     print('wradlib: Calling module <zonalstats> as main...')
